// HttpFilter.cpp : Implementation of CHttpFilter
#include "stdafx.h"
#include "Pluggable.h"
#include "HttpFilter.h"

/////////////////////////////////////////////////////////////////////////////
// CDaveProtocol

static bool bFirstTime;

/////////////////////////////////////////////////////////////////////////////
// CHttpFilter

/////////////////////////////////////////////////////////////////////////////
// IInternetProtocolRoot methods.
/////////////////////////////////////////////////////////////////////////////

// Starts the class associated with the asynchronous pluggable protocol.

HRESULT STDMETHODCALLTYPE CHttpFilter::Start(
	/* [in] */ LPCWSTR szUrl, 
	/* [in] */ IInternetProtocolSink __RPC_FAR  *pOIProtSink,
    /* [in] */ IInternetBindInfo __RPC_FAR *pOIBindInfo,
    /* [in] */ DWORD grfPI,
    /* [in] */ DWORD dwReserved)
{
	USES_CONVERSION;
	const char *	AgentName = "Pluggable";
	const char *	Url = W2CA (szUrl);

	ATLTRACE(_T("CHttpFilter::Start ()\n"));

	bFirstTime = true;
	
	// Save pointers, we'll need them later

	m_pProtocolSink = pOIProtSink;
	m_pBindInfo = pOIBindInfo;

	// Get handle for internet I/O

	m_hInternetSession = InternetOpen (AgentName, INTERNET_OPEN_TYPE_DIRECT, NULL, NULL, INTERNET_FLAG_ASYNC);
	if (!m_hInternetSession) {
		ATLTRACE(_T("InternetOpen () failed\n"));
		return S_FALSE;
	}
	else 
		ATLTRACE(_T("InternetOpen () succeeded, handle is 0x%lX\n"), m_hInternetSession);

	// Want some events, so set the callback up

	InternetSetStatusCallback (m_hInternetSession, CHttpFilter::StatusCallback);

	// Go get the URL... call us when you're done..

#define CANNED_URL "http://ukinterweb.uki.ime.reuters.com/Whats/NovItems/nsg.htm"

	InternetOpenUrl(m_hInternetSession, CANNED_URL, NULL, 0, INTERNET_FLAG_RELOAD, (DWORD) this);

	return S_OK;
}

// Allows the pluggable protocol handler to continue processing data on the apartment 
// (or user interface) thread. This method is called in response to a call to 
// IInternetProtocolSink::Switch.

HRESULT STDMETHODCALLTYPE CHttpFilter::Continue( 
    /* [in] */ PROTOCOLDATA __RPC_FAR *pProtocolData)
{
	ATLTRACE(_T("CHttpFilter::Continue ()\n"));
	return S_OK;
}

// Aborts an operation in progress.

HRESULT STDMETHODCALLTYPE CHttpFilter::Abort( 
    /* [in] */ HRESULT hrReason,
    /* [in] */ DWORD dwOptions)
{
	ATLTRACE(_T("CHttpFilter::Abort ()\n"));

	// Remove Callback

	InternetSetStatusCallback (m_hInternetSession, NULL);

	// Cleanup handles..

	if (m_hHttpSession) {
		InternetCloseHandle (m_hHttpSession);
		m_hHttpSession = 0;
	}

	if (m_hInternetSession) {
		InternetCloseHandle (m_hInternetSession);
		m_hInternetSession = 0;
	}

	return S_OK;
}

// Releases the resources used by the pluggable protocol handler.

HRESULT STDMETHODCALLTYPE CHttpFilter::Terminate( 
    /* [in] */ DWORD dwOptions)
{
	ATLTRACE(_T("CHttpFilter::Terminate ()\n"));
	return S_OK;	
}

/////////////////////////////////////////////////////////////////////////////
// IInternetProtocol based on IInternetProtocolRoot
/////////////////////////////////////////////////////////////////////////////

// Reads data retrieved by the pluggable protocol handler.

HRESULT STDMETHODCALLTYPE CHttpFilter::Read( 
    /* [length_is][size_is][out][in] */ void __RPC_FAR *pv,
    /* [in] */ ULONG cb,
    /* [out] */ ULONG __RPC_FAR *pcbRead)
{
	BOOL	Status;

	Status = InternetReadFile (m_hHttpSession, pv, cb, pcbRead);

	ATLTRACE(_T("nBuffer is %ld\n"), *pcbRead);

	if (Status == FALSE) {
		DWORD error = GetLastError();
		ATLTRACE(_T("Errno = %d\n"), error);
		return INET_E_DOWNLOAD_FAILURE;
	}
	else
	if (*pcbRead == cb) {	// more to read ?
		return S_OK;
	}

	return S_FALSE;	
}

// Moves the current seek offset.

HRESULT STDMETHODCALLTYPE CHttpFilter::Seek( 
    /* [in] */ LARGE_INTEGER dlibMove,
    /* [in] */ DWORD dwOrigin,
    /* [out] */ ULARGE_INTEGER __RPC_FAR *plibNewPosition)
{
	ATLTRACE(_T("CHttpFilter::Seek ()\n"));
	return S_OK;
}

// Locks the request so that IInternetProtocolRoot::Terminate () can be called and the 
// remaining data can be read.

HRESULT STDMETHODCALLTYPE CHttpFilter::LockRequest( 
    /* [in] */ DWORD dwOptions)
{
	ATLTRACE(_T("CHttpFilter::LockRequest ()\n"));
	return S_OK;
}

// Frees any resources associated with a lock. Called only if 
// IInternetProtocol::LockRequest () was called.

HRESULT STDMETHODCALLTYPE CHttpFilter::UnlockRequest( void)
{
	ATLTRACE(_T("CHttpFilter::UnlockRequest ()\n"));
	return S_OK;
}

void __stdcall CHttpFilter::StatusCallback (HINTERNET hSession, DWORD Context, DWORD Status, LPVOID pInformation, DWORD InfoLength)
{
	CHttpFilter *	pClass = (CHttpFilter *) Context;

//	ATLTRACE(_T("StatusCallback (hSession = 0x%lX, Context = 0x%lX, Status = %ld, pInformation = 0x%lX, InfoLength = %ld)\n"),
//		hSession, Context, Status, pInformation, InfoLength);
	switch (Status) {
	case INTERNET_STATUS_RESOLVING_NAME:
		ATLTRACE(_T("INTERNET_STATUS_RESOLVING_NAME\n"));
		break;
	case INTERNET_STATUS_NAME_RESOLVED:
		ATLTRACE(_T("INTERNET_STATUS_NAME_RESOLVED\n"));
		break;
	case INTERNET_STATUS_CONNECTING_TO_SERVER:
		ATLTRACE(_T("INTERNET_STATUS_CONNECTING_TO_SERVER\n"));
		break;
	case INTERNET_STATUS_CONNECTED_TO_SERVER:
		ATLTRACE(_T("INTERNET_STATUS_CONNECTED_TO_SERVER\n"));
		break;
	case INTERNET_STATUS_SENDING_REQUEST:
		ATLTRACE(_T("INTERNET_STATUS_SENDING_REQUEST\n"));
		break;
	case INTERNET_STATUS_REQUEST_SENT:
		ATLTRACE(_T("INTERNET_STATUS_REQUEST_SENT\n"));
		break;
	case INTERNET_STATUS_RECEIVING_RESPONSE:
		ATLTRACE(_T("INTERNET_STATUS_RECEIVING_RESPONSE\n"));
		break;
	case INTERNET_STATUS_RESPONSE_RECEIVED:
		ATLTRACE(_T("INTERNET_STATUS_RESPONSE_RECEIVED (%ld bytes)\n"), (*(DWORD *)pInformation));
		break;
	case INTERNET_STATUS_CTL_RESPONSE_RECEIVED:
		ATLTRACE(_T("INTERNET_STATUS_CTL_RESPONSE_RECEIVED\n"));
		break;
	case INTERNET_STATUS_PREFETCH:
		ATLTRACE(_T("INTERNET_STATUS_PREFETCH\n"));
		break;
	case INTERNET_STATUS_CLOSING_CONNECTION:
		ATLTRACE(_T("INTERNET_STATUS_CLOSING_CONNECTION\n"));
		break;
	case INTERNET_STATUS_CONNECTION_CLOSED:
		ATLTRACE(_T("INTERNET_STATUS_CONNECTION_CLOSED\n"));
		break;
	case INTERNET_STATUS_HANDLE_CREATED:
		ATLTRACE(_T("INTERNET_STATUS_HANDLE_CREATED as 0x%lX\n"), *(DWORD *)pInformation);//hSession);
		pClass -> m_hHttpSession = * (HINTERNET *) pInformation;	// Copy
		break;
	case INTERNET_STATUS_HANDLE_CLOSING:
		ATLTRACE(_T("INTERNET_STATUS_HANDLE_CLOSING\n"));
		break;
	case INTERNET_STATUS_REQUEST_COMPLETE:
		ATLTRACE(_T("INTERNET_STATUS_REQUEST_COMPLETE\n"));

		// Tell sink that we have the data

		pClass -> m_pProtocolSink -> ReportData (BSCF_DATAFULLYAVAILABLE, 100, 100);

		break;
	case INTERNET_STATUS_REDIRECT:
		ATLTRACE(_T("INTERNET_STATUS_REDIRECT\n"));
		break;
	case INTERNET_STATUS_INTERMEDIATE_RESPONSE:
		ATLTRACE(_T("INTERNET_STATUS_INTERMEDIATE_RESPONSE\n"));
		break;
	case INTERNET_STATUS_STATE_CHANGE:
		ATLTRACE(_T("INTERNET_STATUS_STATE_CHANGE\n"));
		break;
	default:
		ATLTRACE(_T("StatusCallback (): Status is %d\n"), Status);
		break;
	}
}

